from rest_framework.decorators import api_view
from rest_framework.response import Response
from .serializers import NoteSerializer
from .models import Note
from owlready2 import *
from rdflib import *

# onto = get_ontology("C:\\Users\\LENOVO\\Desktop\\dogSkinDisease.owl").load(reload_if_newer = True)
# onto = get_ontology(
#     "C:\\Users\\LENOVO\\Music\\Ontology Generation\\Ontology generation\\Fourth\\Ontology with uppercase disease "
#     "name\\dogDisease.owl"
# ).load(
#     reload_if_newer=True)
onto = get_ontology("C:\\Users\\LENOVO\\PycharmProjects\\buildOntology\\dogDisease.owl").load(reload_if_newer=True)
graph = default_world.as_rdflib_graph()


@api_view(['GET'])
def getRoutes(request):
    routes = [
        {
            'EndPoint': '/notes/',
            'method': 'GET',
            'body': None,
            'description': 'Returns an array of notes'
        },
        {
            'EndPoint': '/notes/id',
            'method': 'GET',
            'body': None,
            'description': 'Returns an array of notes'
        },
        {
            'EndPoint': '/decease/treatment/',
            'method': 'POST',
            'body': {'decease': ""},
            'description': 'Get treatments for the decease'
        },
        {
            'EndPoint': '/notes/id/update/',
            'method': 'PUT',
            'body': {'body': ""},
            'description': 'Updates an existing record with data sent in put request'
        },
        {
            'EndPoint': '/notes/id/delete',
            'method': 'DELETE',
            'body': None,
            'description': 'Deletes any existing note'
        }
    ]
    return Response(routes)


@api_view(['GET'])
def getNotes(request):
    notes = Note.objects.all()
    serializer = NoteSerializer(notes, many=True)

    return Response(serializer.data)


@api_view(['GET'])
def getNote(request, pk):
    notes = Note.objects.get(id=pk)
    serializer = NoteSerializer(notes, many=False)
    return Response(serializer.data)


@api_view(['GET'])
def get_disease_description(request, pk):
    disease_description = list(graph.query(
        """
        SELECT ?description 
	    WHERE {  ?disease <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.semanticweb.org/dogdisease/ontologies/2022/4/dogSkinDisease#NamedDisease>;
	    FILTER(?disease = <http://www.semanticweb.org/dogdisease/ontologies/2022/4/dogDisease.owl#"""+pk+""">).
	    ?disease <http://www.semanticweb.org/dogdisease/ontologies/2022/4/dogSkinDisease#hasDescription> ?description.
        }"""
    ))

    new_disease_description = [str(x[0]) for x in disease_description]
    new_disease_description = ' '.join(new_disease_description)
    new_disease_description = new_disease_description.split('.')
    new_disease_description = new_disease_description[1:-1]
    print(new_disease_description)

    return Response(new_disease_description)


@api_view(['GET'])
def get_disease_sub_info(request, pk, pk2, pk3):
    global sci_name

    new_pk2 = pk2[:-1]
    if pk == "Dandruff":
        sci_name = "seborrheic dermatitis"
    elif pk == "Ringworm":
        sci_name = "dermatophytes"
    elif pk == "YeastInfection":
        sci_name = "Malassezia pachydermatitis"

    sub_list_info = list(graph.query(
        """
        SELECT ?treatdescribe
        WHERE {  ?disease <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.semanticweb.org/dogdisease/ontologies/2022/4/dogSkinDisease#NamedDisease>;
        FILTER((?disease = <http://www.semanticweb.org/dogdisease/ontologies/2022/4/dogDisease.owl#""" + pk + """>) && (?treat = <http://www.semanticweb.org/dogdisease/ontologies/2022/4/dogDisease.owl#""" + pk3 + """>)).
        	    ?disease <http://www.semanticweb.org/dogdisease/ontologies/2022/4/dogSkinDisease#has""" + new_pk2 + """> ?treat.
        	    ?treat <http://www.semanticweb.org/dogdisease/ontologies/2022/4/dogSkinDisease#has""" + new_pk2 + """Description> ?treatdescribe.
                }"""))

    new_sub_list_info = [str(x[0]) for x in sub_list_info]
    new_sub_list_info = ' '.join(new_sub_list_info)
    new_sub_list_info = new_sub_list_info.split('.')
    new_sub_list_info = new_sub_list_info[1:-1]
    print(new_sub_list_info)

    return Response(new_sub_list_info)


@api_view(['GET'])
def get_disease_sub(request, pk, pk2):
    new_pk2 = pk2[:-1]

    sub_list = list(graph.query(
        """
        SELECT ?treat
	    WHERE {  ?disease <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.semanticweb.org/dogdisease/ontologies/2022/4/dogSkinDisease#NamedDisease>;
	    FILTER(?disease = <http://www.semanticweb.org/dogdisease/ontologies/2022/4/dogDisease.owl#""" + pk + """>).
	    ?disease <http://www.semanticweb.org/dogdisease/ontologies/2022/4/dogSkinDisease#has""" + new_pk2 + """> ?treat.
        }"""))

    new_sub_list = [str(x[0]) for x in sub_list]
    fin_sub_list = []

    for y in new_sub_list:
        itm = y[y.find('#'):][1:]
        fin_sub_list.append(itm)

    return Response(new_sub_list)


@api_view(['GET'])
def get_diseases(request):
    des_list = list(graph.query("""
                SELECT ?disease
                WHERE {
                ?disease <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.semanticweb.org/dogdisease/ontologies/2022/4/dogSkinDisease#NamedDisease> .
                }"""))
    new_des_list = [str(x[0]) for x in des_list]
    # index = []
    # for y in range(len(new_list)):
    #     index.append(y + 1)
    #
    # print(index)
    # disease_dict = dict.fromkeys(new_list, "disease")
    # index_dict = dict.fromkeys(index, "id")
    # d1 = dict(enumerate(new_list))
    # print(disease_dict)
    # print(index_dict)
    fin_des_list = []

    for y in new_des_list:
        des = y[y.find('#'):][1:]
        fin_des_list.append(des)

    print(fin_des_list)
    return Response(fin_des_list)


@api_view(['POST'])
def get_treatment(request):
    data = request.data
    disease = data['disease']
    res = list(graph.query("""
                SELECT ?disease ?treat ?treatdescribe 
	            WHERE {  ?disease <http://www.semanticweb.org/dogdisease/ontologies/2022/4/dogSkinDisease#hasScientificName> ?o;
	            FILTER(REGEX(?o, \"""" + disease + """\", "i")).
	            ?disease <http://www.semanticweb.org/dogdisease/ontologies/2022/4/dogSkinDisease#hasTreatment> ?treat.
	            ?treat <http://www.semanticweb.org/dogdisease/ontologies/2022/4/dogSkinDisease#hasTreatmentDescription> ?treatdescribe.
                }"""))
    return Response(res)


@api_view(['PUT'])
def updateNote(request, pk):
    data = request.data

    note = Note.objects.get(id=pk)
    serializer = NoteSerializer(note, data=request.data)
    if serializer.is_valid():
        serializer.save()
    return Response(serializer.data)


@api_view(['POST'])
def createNote(request):
    data = request.data

    note = Note.objects.create(
        body=data['body']
    )
    serializer = NoteSerializer(note, many=False)

    return Response(serializer.data)


@api_view(['DELETE'])
def deleteNote(request, pk):
    data = request.data

    note = Note.objects.get(id=pk)
    note.delete()
    return Response("Note has been deleted!")
