# nltk.download('punkt')
# nltk.download('stopwords')
# nltk.download('averaged_perceptron_tagger')
# nltk.download('wordnet')
# nltk.download('omw-1.4')
# import fasttext
import string
import contractions
import nltk
import pandas as pd
from matplotlib import pyplot as plt
from nltk.corpus import stopwords, wordnet
from nltk.stem import WordNetLemmatizer
from nltk.tokenize import word_tokenize

plt.xticks(rotation=70)
pd.options.mode.chained_assignment = None
pd.set_option('display.max_colwidth', 100)

# read file
with open('items.csv') as f:
    df = pd.read_csv(f)
f.close()

# check for missing values
for col in df.columns:
    print(col, df[col].isnull().sum())

rws = df.loc[:, ['Name', 'Info']]

# expanding contractions “I’ve” = “I have”
rws['no_contract'] = rws['Info'].apply(lambda x: [contractions.fix(word) for word in x.split()])
rws.head()

# rejoin words and build string
rws['Info_str'] = [' '.join(map(str, l)) for l in rws['no_contract']]
rws.head()

# split each individual word into a token
rws['tokenized'] = rws['Info_str'].apply(word_tokenize)
rws.head()

# converting all characters to lowercase
rws['lower'] = rws['tokenized'].apply(lambda x: [word.lower() for word in x])
rws.head()

# removing punctuation
punc = string.punctuation
rws['no_punc'] = rws['lower'].apply(lambda x: [word for word in x if word not in punc])
rws.head()

# removing stopwords "you, he, she, in, a, has, are, etc."
stop_words = set(stopwords.words('english'))
rws['stopwords_removed'] = rws['no_punc'].apply(lambda x: [word for word in x if word not in stop_words])
rws.head()

# apply parts of speech "noun, verb, adverb, etc."
rws['pos_tags'] = rws['stopwords_removed'].apply(nltk.tag.pos_tag)
rws.head()


# NLTK’s word lemmatizer which needs the parts of speech tags to be converted to wordnet’s format
def get_wordnet_pos(tag):
    if tag.startswith('J'):
        return wordnet.ADJ
    elif tag.startswith('V'):
        return wordnet.VERB
    elif tag.startswith('N'):
        return wordnet.NOUN
    elif tag.startswith('R'):
        return wordnet.ADV
    else:
        return wordnet.NOUN


rws['wordnet_pos'] = rws['pos_tags'].apply(lambda x: [(word, get_wordnet_pos(pos_tag)) for (word, pos_tag) in x])
rws.head()

# apply NLTK’s word lemmatizer to get the first form of word
wnl = WordNetLemmatizer()
rws['lemmatized'] = rws['wordnet_pos'].apply(lambda x: [wnl.lemmatize(word, tag) for word, tag in x])
rws.head()

# f = lambda x: ', '.join(set([y for y in x.split() if y in s]))
# rws['Keyword1'] = rws['lemmatized'].apply(f)
# rws['Keyword1'] = rws['lemmatized'].apply(lambda x: ', '.join(set(x.split()).intersection(s)))
# rws['filtered'] = rws['lemmatized'].str.contains('allergic' or 'fungus').head()

# filter based on keywords to identify the infection type
keywords = ['allergic', 'infection', 'fungus', 'viral', 'environmental', 'flea', 'mite', 'yeast', 'allergy', 'dandruff', 'environment', 'fleas']
s = set(keywords)
rws['Infection'] = rws['Info'].str.findall('|'.join(keywords)).apply(set).str.join(', ')

# remove columns
# rws.drop('pos_tags', axis=1, inplace=True)
# rws.drop('tokenized', axis=1, inplace=True)
# rws.drop('Info_str', axis=1, inplace=True)
# rws.drop('wordnet_pos', axis=1, inplace=True)
# rws.drop('stopwords_removed', axis=1, inplace=True)
# rws.drop('no_punc', axis=1, inplace=True)
# rws.drop('lower', axis=1, inplace=True)
# rws.drop('no_contract', axis=1, inplace=True)
# rws.drop('Info', axis=1, inplace=True)

# print output to csv
rws.to_csv('pre_processed_disease_infection1.csv')
