from scrapy import Spider
from scrapy.http import Request


class DogdiseasesSpider(Spider):
    name = 'dogdiseases'
    allowed_domains = ['petmd.com']
    start_urls = ['https://www.petmd.com/dog/conditions']

    def __init__(self, disease=None):
        self.disease = disease

    def parse(self, response):
        if self.disease:
            disease_url = response.xpath('//*[contains(@href,"' + self.disease + '")]/@href').extract_first()
            yield Request(response.urljoin(disease_url), callback=self.parse_disease)
        else:
            self.logger.info('Scraping all articles.')
            diseases = response.xpath('//*[@class="field-content"]/a/@href').extract()
            for disease in diseases:
                yield Request(response.urljoin(disease), callback=self.parse_disease)

    def parse_disease(self, response):
        disease_name = response.xpath('//title/text()').extract_first()
        disease_name = disease_name.split(' | ')
        disease_name = disease_name[0]

        topics = response.xpath('//*[@class="paragraph paragraph--type--article-section paragraph--view-mode--default"]')
        for topic in topics:
            topic_heading = topic.xpath('.//h2/text()').extract_first()
            topic_description = topic.xpath('.//*[@class="clearfix text-formatted field field--name-field-section-text field--type-text-long field--label-hidden field__item"]/p/text()').extract()

            yield {
                'disease_name': disease_name,
                'topic_heading': topic_heading,
                'topic_description': topic_description
            }

